<?php

namespace Drupal\fpdi_print;

use Drupal\Component\Utility\Html;
use setasign\Fpdi\Tcpdf\Fpdi;

/**
 * The print builder service.
 */
class PrintBuilder {

  public function getPDF($positions = '', $filePdfTemplate = ''){
    $pdf = new Fpdi();
    $pdf->setPrintHeader(false);
    if(!empty($filePdfTemplate) && file_exists($filePdfTemplate)){
      $pageCount = $pdf->setSourceFile($filePdfTemplate);
      // import page 1
      foreach (range(1, $pageCount) as $page) {
        $templateId = $pdf->importPage($page);
        // get the size of the imported page
        $size = $pdf->getTemplateSize($templateId);

        // create a page (landscape or portrait depending on the imported page size)
        if ($size[0] > $size[1]) {
          $pdf->AddPage('L', array($size[0], $size[1]));
        } else {
          $pdf->AddPage('P', array($size[0], $size[1]));
        }

        // use the imported page
        $pdf->useTemplate($templateId);
        if(!empty($positions[$page])){
          foreach ($positions[$page] as $position){
            if (!empty(array_intersect(array_keys($position),["text","image","html"]))) {
              if (!empty($position['text'])) {
                $position['text'] = $this->formatText($position['text']);
                $pdf->SetXY($position['x'], $position['y']);
                if (!empty($position['height'])) {
                  $pdf->MultiCell(0, $position['height'], $position['text']);
                } else {
                  $pdf->Write(0, $position['text']);
                }
              }
              if (!empty($position['image'])) {
                $position['image'] = $this->formatImage($position['image']);
                [$img_width, $img_height] = getimagesize($position['image']);
                $path_parts = pathinfo($position['image']);
                if (!empty($position['width'])) {
                  $img_width = $position['width'];
                  $img_height = $position['height'];
                }
                $pdf->Image($position['image'], $position['x'], $position['y'], $img_width, $img_height, strtolower($path_parts['extension']));
              }
              if (!empty($position['html'])) {
                $pdf->SetXY($position['x'], $position['y']);
                if (!empty($position['height'])) {
                  $pdf->MultiCell(0, $position['height'], $pdf->WriteHTML($position['text']));
                } else {
                  $pdf->writeHTML($position['html'], true, 0, true, 0);
                }
              }
            }
          }
        }
      }
    }
    return $pdf;
  }
  /**
   * filter value
   */
  protected function formatText($value) {
    $value = Html::decodeEntities($value);
    $value = preg_replace('/<br(\s+\/)/', "\n", $value);
    $value = trim(strip_tags($value));
    return $value;
  }

  /**
   * get image path
   */
  protected function formatImage($img) {
    $img = Html::decodeEntities($img);
    if (is_numeric($img)) {
      $file = \Drupal\file\Entity\File::load($img);
      $uri = $file->getFileUri();
      $stream_manager = \Drupal::service('stream_wrapper_manager')
        ->getViaUri($uri);
      $filename = $stream_manager->realpath();
    }
    else {
      $img = trim(strip_tags($img, '<img>'));
      $doc = new \DOMDocument();
      $doc->loadHTML($img);
      $xpath = new \DOMXPath($doc);
      $src = $xpath->evaluate("string(//img/@src)");
      if (!empty($src)) {
        $img = $src;
      }
      //remove url absolute
      $url = \Drupal\Core\Url::fromUserInput('/', ['absolute' => TRUE])->toString();
      $img = str_replace($url, '', $img);
      $urlParse = parse_url($img);
      //Convert img to path
      $filename = \Drupal::root() . $urlParse['path'];
    }
    $is_picture = is_file($filename) && is_readable($filename) && getimagesize($filename);
    if($is_picture){
      return $filename;
    }
    return FALSE;
  }
}
